/*
 * Copyright (c) 2005, The Black Sheep, Department of Computer Science, The University of Auckland
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of The Black Sheep, The Department of Computer Science or The University of Auckland nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 Contributors and list of changes:

 Cameron Skinner
   Converted original Robocup Rescue software into librescue
*/

// Config.cc
//
/////////////////////////////////////////////////////////////////////////////

#include "config.h"
#include "error.h"
#include <stdio.h>
#include <string>

namespace Librescue {
  /////////////////////////////////////////////////////////////////////////
  // Config
    
  Config::~Config() {
  }
    
  Config::Config() {
#       define P(type_, name_, value_) m_##name_ = value_;
#       include "parameters.h"
#       undef P
  }
    
  bool Config::readConfigFile(const char* filename) {
	//	ASSERT(filename);
	FILE* fp = fopen(filename, "r");
	if(!fp) {
	  std::string message = "error: can't open file '";
	  message += filename;
	  message += "'";
	  logError(message.c_str());
	  return false;
	}
        
	const int size = 1024;
	char buffer[size];
	char name[size];
	char value[size];
	int line = 0;
	while(fgets(buffer, size, fp) != NULL) {
	  line++;
	  if (buffer[0] == '#' || buffer[0] == '\n')
		continue;
            
	  char *p = buffer;
	  while(*p != '\0') {
		if(*p == ':')
		  *p = ' ';
		p++;
	  }
	  int n = sscanf(buffer, "%s %s", name, value);
	  bool valid = false;
	  if(n == 2) {
		if(setValue(name, value))
		  valid = true;
	  }
	  if(!valid)
		fprintf(stderr, "warning: %s (%d) : %s\n", filename, line, buffer);
	}
        
	fclose(fp);
        
	return true;
  }
    
  bool Config::setValue(const char* name, const char* value) {
	//	ASSERT(name && value);
	enum Type {
	  v_string,
	  v_int,
	  v_double,
	  v_bool,
	};
        
	Type type;
	void* variable;
#       define P(type_, name_, value_) if(strcmp(name, #name_) == 0) { type = v_##type_; variable = &m_##name_; goto hit; }
#       include "parameters.h"
	{ return false; }
#       undef P
  hit:
        
	int scanned;
	switch(type) {
	case v_string:
	  scanned = true;
	  *(std::string*)variable = value;
	  break;
	case v_int:
	  scanned = sscanf(value, "%d", (int*)variable);
	  if(scanned < 1 || scanned == EOF)
		return false;
	  break;
	case v_double:
	  scanned = sscanf(value, "%lf", (double*)variable);
	  if(scanned < 1 || scanned == EOF)
		return false;
	  break;
	case v_bool:
	  *(bool*)variable = (value == NULL || (strcmp(value, "off") != 0 && strcmp(value, "false") != 0));
	  break;
	default:
	  break;
	  //	  ASSERT(false);
	}
        
	return true;
  }

  /////////////////////////////////////////////////////////////////////////
} // namespace Rescue
